import React, { useState } from 'react';
import Box from '@mui/material/Box';
import Tab from '@mui/material/Tab';
import TabContext from '@mui/lab/TabContext';
import TabList from '@mui/lab/TabList';
import TabPanel from '@mui/lab/TabPanel';
import TextField from '@mui/material/TextField';
import TemplateViewComponent, { changeFieldInArray, checkIsValidByType, Template } from './templateView';
import { Button } from '@mui/material';
import { SpanError } from './spanError';

interface FormData {
    tab1: {
        id: number;
        name: string;
        title: string;
        isValid: boolean;
        fieldErrors: string[];
    };
    tab2: {
        id: number;
        templates: Template[];
        isValid: boolean;
    };
    tabSelectedId: number;
}
const defaultTemplate: Template = { id: 1, type: 0, viewObject: {}, isValid: false, fieldErrors: [] };

export default function MainTabComponent() {
    const [showError, setShowError] = useState(false);
    const [formData, setFormData] = useState<FormData>(() => ({
        tab1: {
            id: 1,
            name: "",
            title: "",
            isValid: false,
            fieldErrors: []
        },
        tab2: {
            id: 2,
            templates: [defaultTemplate],
            isValid: false,
        },
        tabSelectedId: 1,
    }));


    const handleTabChange = (ev: React.SyntheticEvent, tabId: string) => {
        setFormData((prevObj) => ({
            ...prevObj,
            tabSelectedId: parseInt(tabId)
        }));
    };
    const onTab1TxtChange = (event: React.ChangeEvent<HTMLInputElement | HTMLTextAreaElement>) => {
        const fName = event.target.name;
        const fValue = event.target.value;

        let fieldErrors = formData.tab1.fieldErrors;
        if (event.target.required) {
            fieldErrors = changeFieldInArray(fieldErrors, fName, checkIsValidByType(event.target.type, fValue));
        }

        setFormData((prevObj) => ({
            ...prevObj,
            tab1: {
                ...prevObj.tab1,
                [fName]: fValue,
                fieldErrors: fieldErrors,
                isValid: fieldErrors.length === 0
            },
            isValid: fieldErrors.length === 0 && prevObj.tab2.isValid
        }));
    }
    const onSubmit = (e: React.FormEvent<HTMLFormElement>) => {
        e.preventDefault();

        if (!checkFormValid()) {
            setShowError(true);
            alert("Please fill in all required information!");
            return;
        }

        alert("pass!");
    }

    const checkFormValid = () => {
        console.log({ formData })
        //check tab1 object VALID
        let tab1FieldErrors: string[] = [];
        tab1FieldErrors = changeFieldInArray(tab1FieldErrors, "name", checkIsValidByType("text", formData.tab1.name));
        tab1FieldErrors = changeFieldInArray(tab1FieldErrors, "title", checkIsValidByType("text", formData.tab1.title));

        let isValid = tab1FieldErrors.length === 0 && formData.tab2.isValid;
        setFormData((prevObj) => ({
            ...prevObj,
            tab1: {
                ...prevObj.tab1,
                fieldErrors: tab1FieldErrors,
                isValid: tab1FieldErrors.length === 0
            }
        }));

        return isValid;
    }

    const onUpdateTemplates = (templates: any[]) => {
        setFormData(prevObj => ({
            ...prevObj,
            tab2: {
                ...prevObj.tab2,
                templates: templates,
                isValid: !templates.some(x => !x.isValid)
            }
        }))
    }

    return (
        <Box noValidate component="form" autoComplete="off" onSubmit={onSubmit}>
            <Button variant='outlined' type="submit">Submit</Button>

            <Box sx={{ width: '100%', typography: 'body1' }}>
                <TabContext value={formData.tabSelectedId.toString()}>
                    <Box sx={{ borderBottom: 1, borderColor: 'divider' }}>
                        <TabList onChange={handleTabChange}>
                            <Tab label={formData.tab1.id.toString()} value={formData.tab1.id.toString()} />
                            <Tab label={formData.tab2.id.toString()} value={formData.tab2.id.toString()} />
                        </TabList>
                    </Box>
                    <TabPanel value={formData.tab1.id.toString()}>
                        <table className='table table-hover'>
                            <tbody>
                                <tr>
                                    <td>
                                        <TextField required name='name'
                                            label="Name" value={formData.tab1.name} onChange={onTab1TxtChange} />
                                        <SpanError isDisplay={showError && formData.tab1.fieldErrors.includes('name')} />
                                    </td>
                                </tr>
                                <tr>
                                    <td>
                                        <TextField required name='title'
                                            label="Title" value={formData.tab1.title} onChange={onTab1TxtChange} />
                                        <SpanError isDisplay={showError && formData.tab1.fieldErrors.includes('title')} />
                                    </td>
                                </tr>
                            </tbody>
                        </table>


                    </TabPanel>
                    <TabPanel value={formData.tab2.id.toString()}>
                        <TemplateViewComponent
                            showError={showError}
                            templates={formData.tab2.templates}
                            onUpdateTemplates={onUpdateTemplates}
                        />
                    </TabPanel>
                </TabContext>
            </Box>
        </Box>
    );
}
