import { Box, Button, Checkbox, MenuItem, Select, SelectChangeEvent, TextField } from "@mui/material";
import { useEffect, useState } from "react";
import { SpanError } from "./spanError";

export const changeFieldInArray = (fields: string[], fieldName: string, isRemove: boolean = false) => {
    const idx = fields.indexOf(fieldName);
    if (isRemove) {
        if (idx !== -1)
            fields.splice(idx, 1);
    }
    else if (idx === -1)
        fields.push(fieldName);
    return fields;
}
export const checkIsValidByType = (txtFieldType: string, txtValue: any) => {
    if (txtValue.length === 0) return false;
    switch (txtFieldType) {
        case "number":
            return (parseInt(txtValue) || 0) > 0;
        case "email":
            const re = /\S+@\S+\.\S+/;
            return re.test(txtValue);
        case "checkbox":
            return txtValue as boolean;
    }
    return true;
}

export interface Template {
    id: number;
    type: number;
    viewObject: any;
    isValid: boolean;
    fieldErrors: string[];
}
interface Props {
    templates: Template[];
    onUpdateTemplates: (templates: Template[]) => void;
    showError: boolean;
}

interface LocalData {
    templates: Template[];
    currentTemplate: Template;
    viewCount: number;
    currentView: number;
    isValid: boolean;
}
const fieldTypes = [
    {
        value: 0,
        title: "--none--",
    },
    {
        value: 1,
        title: "Template 1",
    },
    {
        value: 2,
        title: "Template 2",
    }
]

const TemplateViewComponent = (props: Props) => {
    const { templates, showError, onUpdateTemplates } = props;
    const [localShowError, setLocalShowError] = useState(false);
    const [localData, setLocalData] = useState<LocalData>(() => ({
        currentView: 0,
        templates: templates,
        currentTemplate: templates[0],
        viewCount: templates.length,
        isValid: false,
    }));

    const changeCurrentTemplate = (template: Template) => {
        setLocalData((prevObj) => {
            const updateTemplates = prevObj.templates.slice();
            const idx = updateTemplates.findIndex(x => x.id === template.id);

            if (idx !== -1)
                updateTemplates[idx] = template;
            else
                updateTemplates.push(template);

            return {
                ...prevObj,
                currentView: template.id - 1,
                isValid: prevObj.isValid && template.isValid,
                currentTemplate: template,
                viewCount: updateTemplates.length,
                templates: updateTemplates
            }
        });

    }

    const onClickView = (i: number) => {
        const currentTemplate = localData.templates.find(x => x.id === i + 1) as Template;
        changeCurrentTemplate(currentTemplate);
    }

    const onClickViewAdd = () => {
        //check valid
        if (!checkValidLocalData()) {
            setLocalShowError(true);
            return;
        }

        const template: Template = { id: localData.viewCount + 1, type: fieldTypes[0].value, viewObject: {}, isValid: false, fieldErrors: [] };
        changeCurrentTemplate(template);
    }

    const handleSelectChange = (ev: SelectChangeEvent) => {
        const typeSelected = parseInt(ev.target.value);
        let templateSelected: Template = {
            ...localData.currentTemplate,
            type: typeSelected,
            isValid: false,
        };

        if (typeSelected === 1) {
            templateSelected.viewObject = { email: "", gender: false };
        }
        else if (typeSelected === 2) {
            templateSelected.viewObject = { id: 0, username: "", password: "" };
        }
        else {
            templateSelected.viewObject = {};
        }
        changeCurrentTemplate(templateSelected);
    };

    const handleFieldChange = (event: React.ChangeEvent<HTMLInputElement>) => {
        const fieldName = event.target.name;
        const fieldType = event.target.type;
        const fieldValue = fieldType === "checkbox" ? event.target.checked : event.target.value;
        
        let fieldErrors = localData.currentTemplate.fieldErrors;
        if (event.target.required) {
            fieldErrors = changeFieldInArray(fieldErrors, fieldName, checkIsValidByType(fieldType, fieldValue));
        }
        const currentTemplate: Template = {
            ...localData.currentTemplate,
            fieldErrors: fieldErrors,
            isValid: fieldErrors.length === 0,
            viewObject: {
                ...localData.currentTemplate.viewObject,
                [fieldName]: fieldValue
            }
        };
        changeCurrentTemplate(currentTemplate);
    }
    const checkValidLocalData = () => {
        let isValid = true;
        const updateTemplates = localData.templates.slice();
        updateTemplates.forEach((template, i) => {
            let fieldErrors: string[] = [];
            if (template.type === 1) {
                fieldErrors = changeFieldInArray(fieldErrors, "email", checkIsValidByType("email", template.viewObject.email));
            }
            else if (template.type === 2) {
                fieldErrors = changeFieldInArray(fieldErrors, "id", checkIsValidByType("number", template.viewObject.id));
                fieldErrors = changeFieldInArray(fieldErrors, "username", checkIsValidByType("text", template.viewObject.username));
                fieldErrors = changeFieldInArray(fieldErrors, "password", checkIsValidByType("text", template.viewObject.password));
            }
            template.isValid = template.type > 0 && fieldErrors.length === 0;
            template.fieldErrors = fieldErrors;
            isValid = isValid && template.isValid;
        });
        setLocalData(prevObj => ({
            ...prevObj,
            templates: updateTemplates,
            isValid: isValid
        }))
        return isValid;
    }

    const renderButtonView = () => {
        const btns = [];
        for (let i = 0; i < localData.viewCount; i++) {
            btns.push(i)
        }
        return (
            btns.map((i) => {
                return <Button
                    style={localData.currentView === i ? { color: "red", border: "solid 1px red" } : {}}
                    key={"btnv_" + i}
                    onClick={() => onClickView(i)}
                >{"View " + i}</Button>;
            }));
    }

    useEffect(() => {
        setLocalShowError(showError);
        if (showError)
            checkValidLocalData();
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [showError]);

    useEffect(() => {
        onUpdateTemplates(localData.templates);
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [localData.templates]);


    const BuildTemplate = () => {
        if (localData.currentTemplate.type === 1) {
            return <table className="table table-hover">
                <tbody>
                    <tr>
                        <td>Email</td>
                        <td>
                            <TextField required name="email"
                                type="email" value={localData.currentTemplate.viewObject.email} onChange={handleFieldChange} />

                            <SpanError isDisplay={localShowError && localData.currentTemplate.fieldErrors.includes('email')} />
                        </td>
                    </tr>
                    <tr><td>Age</td><td>
                        <TextField name="age" type="number" value={localData.currentTemplate.viewObject.age} onChange={handleFieldChange} /></td>
                    </tr>
                    <tr>
                        <td>Gender</td>
                        <td><Checkbox name="gender" checked={localData.currentTemplate.viewObject.gender} onChange={handleFieldChange} /></td>
                    </tr>
                </tbody>
            </table>
        }
        else if (localData.currentTemplate.type === 2)
            return <table className="table table-hover">
                <tbody>
                    <tr>
                        <td>Id</td>
                        <td>
                            <TextField required name="id"
                                type="number" value={localData.currentTemplate.viewObject.id} onChange={handleFieldChange} />

                            <SpanError isDisplay={localShowError && localData.currentTemplate.fieldErrors.includes('id')} />
                        </td>
                    </tr>
                    <tr><td>Username</td><td>
                        <TextField required name="username"
                            value={localData.currentTemplate.viewObject.username} onChange={handleFieldChange} />
                        <SpanError isDisplay={localShowError && localData.currentTemplate.fieldErrors.includes('username')} />
                    </td>
                    </tr>
                    <tr>
                        <td>Password</td>
                        <td>
                            <TextField required name="password"
                                type="password" value={localData.currentTemplate.viewObject.password} onChange={handleFieldChange} />
                            <SpanError isDisplay={localShowError && localData.currentTemplate.fieldErrors.includes('password')} />
                        </td>
                    </tr>
                </tbody>
            </table>

        return <></>;
    }
    return (
        <>
            {renderButtonView()}
            <Button onClick={onClickViewAdd}>+</Button>

            <Box>
                Template:<br />
                <Select required
                    value={localData.currentTemplate.type.toString()}
                    onChange={handleSelectChange}
                >
                    {fieldTypes.map((type) => (
                        <MenuItem value={type.value} key={type.value}>
                            {type.title}
                        </MenuItem>
                    ))}
                </Select>
                <SpanError isDisplay={localShowError && localData.currentTemplate.type === 0} />

                {BuildTemplate()}
            </Box>
        </>
    )
}
export default TemplateViewComponent;